<?php

namespace Drupal\iconify_icons\Plugin\Field\FieldFormatter;

use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\iconify_icons\IconifyServiceInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Implementation of Iconify icon formatter.
 *
 * @FieldFormatter(
 *   id = "iconify_icon_formatter",
 *   label = @Translation("Iconify Icon"),
 *   field_types = {
 *     "iconify_icon"
 *   }
 * )
 */
class IconifyIconFormatter extends FormatterBase implements ContainerFactoryPluginInterface {

  /**
   * Drupal configuration service container.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * Drupal Iconify service.
   *
   * @var \Drupal\iconify_icons\IconifyServiceInterface
   */
  protected $iconify;

  /**
   * {@inheritdoc}
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, $label, $view_mode, array $third_party_settings, ConfigFactory $config_factory, IconifyServiceInterface $iconify) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);

    $this->configFactory = $config_factory;
    $this->iconify = $iconify;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('config.factory'),
      $container->get('iconify_icons.iconify_service'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    // Early opt-out if the field is empty.
    if (count($items) <= 0) {
      return [];
    }

    $iconify_icons = $items->getValue();

    // Loop over each icon and build data.
    $icons = [];
    foreach ($iconify_icons as $icon) {
      $settings = unserialize($icon['settings'] ?? '', ['allowed_classes']);
      $icons[] = [
        '#type' => 'iconify_icon',
        '#icon' => $icon['icon'],
        '#settings' => $settings,
      ];
    }

    return [
      [
        '#theme' => 'iconify_icons',
        '#icons' => $icons,
        '#attached' => [
          'library' => [
            'iconify_icons/default',
          ],
        ],
      ],
    ];
  }

}
